#' Herleitung der Spezialfällelimitierung nach der NEK-Methode Version 2023
#'
#' Mit der NEK werden Standorte hinsichtlich ihrer landwirtschaftlichen Nutzungseignung beurteilt.
#' Dabei werden Boden, Klima und Terrain berücksichtigt.
#' Dieses Beurteilungsschema wurde hier publiziert:
#'
#' Greiner, L.; Petter, G.; Keller, A. (2023): Anleitung zur Beurteilung der
#' landwirtschaftlichen Nutzungseignung. Methode der Nutzungseignungsklassen (NEK-Methode).
#' Aktualisierte Version 2023. Kompetenzzentrum Boden KOBO. BFH-HAFL.
#'
#'
#' Die hier vorliegende Beurteilung basiert
#' auf den Bewertungstabellen der NEK-2023 Stand 24.10.2023
#'
#'
#'
#' @param hangneigung Hangneigung
#' @param whug Wasserhaushaltsuntergruppe nach FAL24
#' @param neigungsart_unregelmaessig ungleichmässige Neigungsart handelt (1) oder nicht (0), abgeleitet aus der Geländeform nach FAL24
#' @param ub_ton Tongehalt im Unterboden (% minerlische Feinerde)
#' @param ub_schluff Schluffgehalt im Unterboden (% minerlische Feinerde)
#' @param klimalim Klimatische Limitierung hergeleitet aus den Klimaeignungszonen
#' @param bewertungstab Bewertungstabelle mit der Zurordung des Terrains, der Wasserhaushalts und der Unterbodentextur zu einer NEK nach der klimatischer Limitierung unterteilt
#' @return Die NEK, welche sich aus der Limitierung durch die Spezialfälle 1 und 2 und das Klima ergibt
#' @export
#'
#'
nek_spezialfaelle<- function(hangneigung, whg, neigungsart_unregelmaessig,
                             ub_ton, ub_schluff, klimalim, bewertungstab_whg_hn, bewertungstab_whg_hn_texturUB){

  # Spezialfall WHG und Hangneigung------------------------------
  startspalte_nek <- 5
  anz_profile<-length(klimalim)

  index_interaktion <- array(NaN,dim=c(anz_profile,1))
  for(i in 1:anz_profile){
    if(!is.nan(whg[i]) &
       !is.nan(hangneigung[i])& !is.nan(neigungsart_unregelmaessig[i])){

      index_temp_hangneigung <- which(hangneigung[i]>=bewertungstab_whg_hn[,2] &
                                        hangneigung[i]<=bewertungstab_whg_hn[,3])
      index_temp_whg     <- which(whg[i]==bewertungstab_whg_hn[,1])
      index_neigungsart_temp <- which(neigungsart_unregelmaessig[i] == bewertungstab_whg_hn[,4])


      index_col <- klimalim[i] + startspalte_nek-1
      if (!is.na(klimalim[i]) & klimalim[i]<=4){
        common_index <- Reduce(intersect, list(index_temp_hangneigung ,index_temp_whg,index_neigungsart_temp))

        if (length(common_index)>0){
          Bewertungen <-bewertungstab_whg_hn[common_index, index_col]
          if(!all(is.na(Bewertungen))){
            index_interaktion[i]<-common_index[min(which(Bewertungen==min(Bewertungen, na.rm = TRUE)))]}
        }
      }
    }
  }

  # NEK entsprechend der Interaktionen zuordnen
  nek_spez1 <- array(NaN,dim=c(anz_profile,1))

  for(i in 1:anz_profile){
    if (!is.na(klimalim[i]) & klimalim[i]<=4){
      index_col <- klimalim[i] + startspalte_nek-1
    } else {
      index_col <- NaN
    }

    if (!is.nan(index_col) & !is.nan(index_interaktion[i] & !is.na(index_interaktion[i]) &
                                     !is.na(index_col) & !is.na(index_interaktion[i]))){
      nek_spez1[i] <- as.numeric(bewertungstab_whg_hn[index_interaktion[i], index_col]) }
    rm(index_col)
  }




  # Spezialfall WHG und Hangneigung und Textur Unterboden------------------------------
  startspalte_nek <- 9

  index_interaktion <- array(NaN,dim=c(anz_profile,1))
  for(i in 1:anz_profile){
    if(!is.nan(whg[i]) &
       !is.nan(hangneigung[i])& !is.nan(neigungsart_unregelmaessig[i])&
       !is.nan(ub_ton[i]) & !is.nan(ub_schluff[i])){

      index_temp_hangneigung <- which(hangneigung[i]>=bewertungstab_whg_hn_texturUB[,2] &
                                        hangneigung[i]<=bewertungstab_whg_hn_texturUB[,3])
      index_temp_whg     <- which(whg[i]==bewertungstab_whg_hn_texturUB[,1])
      index_neigungsart_temp <- which(neigungsart_unregelmaessig[i] == bewertungstab_whg_hn_texturUB[,4])
      index_textur_temp <- which(ub_ton[i]>=bewertungstab_whg_hn_texturUB[,5] &
                                   ub_ton[i]<=bewertungstab_whg_hn_texturUB[,6]&
                                   ub_schluff[i]>=bewertungstab_whg_hn_texturUB[,7] &
                                   ub_schluff[i]<=bewertungstab_whg_hn_texturUB[,8])


      index_col <- klimalim[i] + startspalte_nek-1
      if (!is.na(klimalim[i]) & klimalim[i]<=4){
        common_index <- Reduce(intersect, list(index_temp_hangneigung ,index_temp_whg,index_neigungsart_temp,index_textur_temp))

        if (length(common_index)>0){
          Bewertungen <-bewertungstab_whg_hn_texturUB[common_index, index_col]
          if(!all(is.na(Bewertungen))){
            index_interaktion[i]<-common_index[min(which(Bewertungen==min(Bewertungen, na.rm = TRUE)))]}
        }
      }
    }
  }

  # NEK entsprechend der Interaktionen zuordnen
  nek_spez2 <- array(NaN,dim=c(anz_profile,1))

  for(i in 1:anz_profile){
    if (!is.na(klimalim[i]) & klimalim[i]<=4){
      index_col <- klimalim[i] + startspalte_nek-1
    } else {
      index_col <- NaN
    }

    if (!is.nan(index_col) & !is.nan(index_interaktion[i] & !is.na(index_interaktion[i]) &
                                     !is.na(index_col) & !is.na(index_interaktion[i]))){
      nek_spez2[i] <- as.numeric(bewertungstab_whg_hn_texturUB[index_interaktion[i], index_col]) }
    rm(index_col)
  }




  # NEK-Spezialfälle herleiten-----------------------
  nek_spezialfaelle <- array(NaN,dim=c(anz_profile,1))

  for(i in 1:anz_profile){
    if(is.na(nek_spez1[i])& is.na(nek_spez2[i])){
      nek_spezialfaelle[i]<-NaN} else {
      nek_spezialfaelle[i] <- max(nek_spez1[i], nek_spez2[i], na.rm = TRUE)
    }
  }



  return(nek_spezialfaelle)

}
